"""
Sentinel-Toolkit
================

`Sentinel-Toolkit <https://github.com/sentinel-toolkit/sentinel-toolkit>`__ is an open-source
`Python <https://www.python.org/>`__ package providing various utility tools for working
with Sentinel-2 satellite images. For example, you can use sentinel_toolkit.S2Srf class
as a python wrapper of a given S2_SRF Excel file to read the spectral responses into ndarray.
"""

from sentinel_toolkit import ecostress
from sentinel_toolkit import srf
from sentinel_toolkit import colorimetry
from sentinel_toolkit import converter
from sentinel_toolkit import product
