"""
Illuminants module provides various illuminant spectral data.
"""

from sentinel_toolkit.colorimetry.illuminants.d65 import D65_360_830_1NM_DISTRIBUTION
from sentinel_toolkit.colorimetry.illuminants.d65 import D65_360_830_1NM_VALUES
