"""
Converter
================

Converter module provides the class EcostressToSentinelConverter
for converting all the examples from the Ecostress spectral library
to Sentinel-2 Responses and writing them to a CSV file.
"""

from sentinel_toolkit.converter.converter import EcostressToSentinelConverter
