"""
S2Product
================

S2Product module provides tools for working with Sentinel-2 product data.

s2_product provides the class S2Product that has methods for working with
band data - for example: read specific band, convert dn to sentinel,
get the metadata.

"""

from sentinel_toolkit.product.s2_product import S2Product
from sentinel_toolkit.product.product_metadata import S2ProductMetadata
