"""
S2Product Metadata
================

S2Product Metadata module provides tools for working with
Sentinel-2 product metadata.

s2_metadata provides the class S2ProductMetadata that has methods
for retrieving information like Sensing Date, NODATA Value,
Quantification Value, BOA Offset Values, Solar Irradiances.
"""

from sentinel_toolkit.product.product_metadata.s2_metadata import S2ProductMetadata
